import sys, os
import wavemqtt
import time
from aenum import Enum, auto
import pint

class ConixPoster:

    def __init__(self, client_id, domain_url = 'stream.conixdb.io', domain_username='conix', domain_password='stream', domain_port='8883'):
        #start up a conix client
        self.client = wavemqtt.Client(str(client_id),
            mosquitto_url=domain_url,
            mosquitto_pass=domain_password,
            mosquitto_user=domain_username,
            mosquitto_port=domain_port,
            mosquitto_tls = True)

        self.registrationMap = {}

    """
    This post sensor data to conix in the right format

    sensor_uuid - some unique string identifying a sensor
    sensor - the type of sensor you are posting. must be of type ConixPoster.SensorTypes
    value - the value of the sensor. Can be number or text
    unit - A string of a standard units. A complete list is here: https://github.com/hgrecco/pint/blob/master/pint/default_en.txt
        Unit also accepts percent, ratio, number and count
    timestamp - microsecond epoch time. Optionally. Excluding will set to now
    """
    def post(self, sensor_uuid, sensor, value, unit, timestamp=None):

        #is this a valid sensor type?
        if not (isinstance(sensor,Sensors) or isinstance(sensor,Diagnostics)):
            raise TypeError("sensor must by a predefined sensor type. Find your sensor/diagnostic type in conixposter.py or add one if it doesn't exist.")

        parsed_unit = None
        if unit == 'ratio': 
            parsed_unit = 'ratio'
        elif unit == 'percent' or unit == '%':
            parsed_unit = 'percent'
        elif unit == 'number' or unit == 'count':
            parsed_unit = 'count'
        elif isinstance(unit, str):
            ureg = pint.UnitRegistry()
            parsed_unit = ureg.parse_units(unit)
        else:
            raise TypeError("Unit must either be a unit string or predefined unit type")

        #do we know about this sensor uuid?
        if sensor_uuid not in self.registrationMap:
            try:
                namespace = self.client.register(sensor_uuid)
                self.registrationMap[sensor_uuid] = namespace
            except TimeoutError as e:
                print(e)
                print("Post failed.")
                print("Maybe the registration server is down?")
                return

        message = {}
        message['uuid'] = sensor_uuid
        message[str(sensor.name)] = value
        message[str(sensor.name) + '_units'] = str(parsed_unit)
        if timestamp is not None:
            message['timestamp'] = timestamp
        else:
            message['timestamp'] = int(round(time.time()*1000000))

        print(message)

        self.client.publish(self.registrationMap[sensor_uuid],sensor_uuid + '/' + str(sensor.name), message)

class Diagnostics(Enum):
       Battery_Energy = auto()
       Battery_Percent = auto()
       Battery_Voltage = auto()
       CPU_Usage = auto()
       Memory_Usage = auto()
       Memory_Total = auto()
       Packet_Count = auto()
       Up_Time = auto()

class Sensors(Enum):
       Zone_Temperature = auto()
       Zone_Humidity = auto()
       Zone_Air_Temperature = auto()
       Yearly_Steam_Usage = auto()
       Wind_Speed = auto()
       Wind_Direction = auto()
       Weather_Wind_Speed = auto()
       Weather_Wind_Direction = auto()
       Weather_Temperature = auto()
       Weather_Solar_Radiance = auto()
       Weather_Relative_Humidity = auto()
       Weather_Rain = auto()
       Weather_Rain_Duration = auto()
       Weather_Outside_Enthalpy = auto()
       Weather_Hail = auto()
       Weather_Cloud_Height = auto()
       Weather_Cloud_Coverage = auto()
       Water_Usage = auto()
       Water_System_Water_Supply_Temperature = auto()
       Water_System_Water_Discharge_Temperature = auto()
       Water_System_Deionised_Water_Level = auto()
       Water_System_Deionised_Water_Conductivity = auto()
       Water_System_DI_Water_Level = auto()
       Water_Supply_Temperature = auto()
       Water_Level = auto()
       Water_Flow = auto()
       Water_Discharge_Temperature = auto()
       Warmest_Zone_Temperature = auto()
       Warm_Cool_Adjust = auto()
       Voltage = auto()
       Velocity_Pressure = auto()
       Valve_Pressure = auto()
       VFD_Speed = auto()
       VFD_Running_Hour = auto()
       VFD_Run_Time = auto()
       VFD_Output_Voltage = auto()
       VFD_Output_Frequency = auto()
       VFD_On_Timer = auto()
       VFD_Motor_Torque = auto()
       VFD_Motor_Speed = auto()
       VFD_Motor_Current = auto()
       VFD_Load_Current = auto()
       VFD_Energy = auto()
       VFD_Drive_Temperature = auto()
       VFD_DC_Bus_Voltage = auto()
       VFD_Current = auto()
       VAV_Zone_Temperature = auto()
       VAV_Zone_Air_Temperature = auto()
       VAV_Warm_Cool_Adjust = auto()
       VAV_Thermostat_Adjust = auto()
       VAV_Supply_Air_Velocity_Pressure = auto()
       VAV_Supply_Air_Temperature = auto()
       VAV_Supply_Air_Flow = auto()
       VAV_Room_Temperature = auto()
       VAV_Return_Air_Temperature = auto()
       VAV_Occupancy = auto()
       VAV_Discharge_Air_Velocity_Pressure = auto()
       VAV_Discharge_Air_Temperature = auto()
       VAV_Discharge_Air_Flow = auto()
       VAV_Damper_Position = auto()
       VAV_CO2 = auto()
       VAV_CO2_Level = auto()
       VAV_Booster_Fan_Air_Flow = auto()
       Usage = auto()
       Underfloor_Temperature = auto()
       Trace_Heat = auto()
       Torque = auto()
       Today_Steam_Usage = auto()
       Timer = auto()
       Thermostat_Adjust = auto()
       Thermal_Power = auto()
       Temperature = auto()
       Supply_Water_Temperature = auto()
       Supply_Water_Flow = auto()
       Supply_Fan_VFD_Speed = auto()
       Supply_Fan_Piezoelectric = auto()
       Supply_Fan_Fan_Air_Flow = auto()
       Supply_Fan_Differential_Pressure = auto()
       Supply_Fan_Cooling_Coil_Valve_Pressure = auto()
       Supply_Fan_Air_Flow = auto()
       Supply_Air_Velocity_Pressure = auto()
       Supply_Air_Temperature = auto()
       Supply_Air_Static_Pressure = auto()
       Supply_Air_Humidity = auto()
       Supply_Air_Flow = auto()
       Steam_Usage = auto()
       Static_Pressure = auto()
       Speed = auto()
       Solar_Zenith_Angle = auto()
       Solar_Radiance = auto()
       Solar_Panel_Solar_Zenith_Angle = auto()
       Solar_Panel_Solar_Azimuth_Angle = auto()
       Solar_Panel_Photovoltaic_Current_Output = auto()
       Solar_Panel_PV_Current_Output = auto()
       Solar_Panel_Battery_Voltage = auto()
       Solar_Azimuth_Angle = auto()
       Running_Hour = auto()
       Run_Time = auto()
       Room_Temperature = auto()
       Return_Water_Temperature = auto()
       Return_Fan_VFD_Speed = auto()
       Return_Fan_Fan_Air_Flow = auto()
       Return_Fan_Differential_Speed = auto()
       Return_Fan_Air_Flow = auto()
       Return_Air_Temperature = auto()
       Return_Air_Humidity = auto()
       Return_Air_Grains = auto()
       Return_Air_Flow = auto()
       Return_Air_Enthalpy = auto()
       Return_Air_Dewpoint = auto()
       Return_Air_CO2 = auto()
       Relative_Humidity = auto()
       Rain = auto()
       Rain_Duration = auto()
       Pressure = auto()
       Preheat_Supply_Air_Temperature = auto()
       Preheat_Discharge_Air_Temperature = auto()
       Preheat_Coil_Leaving_Water_Temperature = auto()
       Preheat_Coil_Entering_Air_Temperature = auto()
       Power_System_Peak_Power_Demand = auto()
       Power = auto()
       Piezoelectric = auto()
       Photovoltaic_Current_Output = auto()
       Peak_Power_Demand = auto()
       PV_Current_Output = auto()
       PIR = auto()
       Outside_Luminance = auto()
       Outside_Enthalpy = auto()
       Outside_Air_Temperature = auto()
       Outside_Air_Lockout_Temperature_Differential = auto()
       Outside_Air_Humidity = auto()
       Outside_Air_Grains = auto()
       Outside_Air_Flow = auto()
       Outside_Air_Enthalpy = auto()
       Outside_Air_Dewpoint = auto()
       Outside_Air_Damper_Position = auto()
       Outside_Air_CO2 = auto()
       Output_Voltage = auto()
       Output_Frequency = auto()
       On_Timer = auto()
       Occupancy = auto()
       Motor_Torque = auto()
       Motor_Speed = auto()
       Motor_Power = auto()
       Motor_Current = auto()
       Motion = auto()
       Monthly_Steam_Usage = auto()
       Mixed_Air_Temperature = auto()
       Mixed_Air_Damper_Position = auto()
       Medium_Temperature_Hot_Water_Supply_Temperature = auto()
       Medium_Temperature_Hot_Water_Return_Temperature = auto()
       Medium_Temperature_Hot_Water_Discharge_Temperature = auto()
       Medium_Temperature_Hot_Water_Differential_Pressure = auto()
       Luminance = auto()
       Lowest_Zone_Temperature = auto()
       Lowest_Exhaust_Air_Static_Pressure = auto()
       Low_Outside_Air_Temperature_Enable_Differential = auto()
       Location_Local_X = auto()
       Location_Local_Y = auto()
       Location_Local_Z = auto()
       Location_GPS_Latitude = auto()
       Location_GPS_Longitude = auto()
       Location_Origin_UUID = auto()
       Load_Current = auto()
       Lighting_System_Luminance = auto()
       Leaving_Water_Temperature = auto()
       Illumination = auto()
       Ice_Tank_Leaving_Water_Temperature = auto()
       Ice_Tank_Entering_Water_Temperature = auto()
       Humidity = auto()
       Hot_Water_Usage = auto()
       Hot_Water_Supply_Temperature = auto()
       Hot_Water_Return_Temperature = auto()
       Hot_Water_Flow = auto()
       Hot_Water_Discharge_Temperature = auto()
       Hot_Water_Differential_Pressure = auto()
       Hot_Water_Coil_Entering_Temperature = auto()
       Hot_Box_Temperature = auto()
       Highest_Zone_Temperature = auto()
       High_Temperature_Hot_Water_Supply_Temperature = auto()
       High_Temperature_Hot_Water_Return_Temperature = auto()
       High_Temperature_Hot_Water_Discharge_Temperature = auto()
       Heating_Thermal_Power = auto()
       Heat_Wheel_Voltage = auto()
       Heat_Wheel_Supply_Air_Temperature = auto()
       Heat_Wheel_Speed = auto()
       Heat_Wheel_Discharge_Air_Temperature = auto()
       Heat_Wheel_Differential_Pressure = auto()
       Heat_Exchanger_Supply_Water_Temperature = auto()
       Heat_Exchanger_Discharge_Water_Temperature = auto()
       Hail = auto()
       HWS_Medium_Temperature_Hot_Water_Supply_Temperature = auto()
       HWS_Medium_Temperature_Hot_Water_Return_Temperature = auto()
       HWS_Medium_Temperature_Hot_Water_Discharge_Temperature = auto()
       HWS_Medium_Temperature_Hot_Water_Differential_Pressure = auto()
       HWS_Hot_Water_Supply_Temperature = auto()
       HWS_Hot_Water_Return_Temperature = auto()
       HWS_Hot_Water_Discharge_Temperature = auto()
       HWS_Hot_Water_Differential_Pressure = auto()
       HWS_Hot_Water_Coil_Entering_Temperature = auto()
       HWS_High_Temperature_Hot_Water_Supply_Temperature = auto()
       HWS_High_Temperature_Hot_Water_Return_Temperature = auto()
       HWS_High_Temperature_Hot_Water_Discharge_Temperature = auto()
       HWS_Heat_Exchanger_Supply_Water_Temperature = auto()
       HWS_Heat_Exchanger_Discharge_Water_Temperature = auto()
       HVAC_Warmest_Zone_Temperature = auto()
       HVAC_Lowest_Zone_Temperature = auto()
       HVAC_Highest_Zone_Temperature = auto()
       HVAC_Coldest_Zone_Temperature = auto()
       HVAC_Average_Zone_Temperature = auto()
       HVAC_Average_Supply_Air_Flow = auto()
       HVAC_Average_Discharge_Air_Flow = auto()
       HVAC_Average_Cooling_Demand = auto()
       Gas_Usage = auto()
       Fume_Hood_Sash_Position = auto()
       Fume_Hood_Air_Flow = auto()
       Frost = auto()
       Frequency = auto()
       Freezer_Temperature = auto()
       Flow = auto()
       Filter_Differential_Pressure = auto()
       Fan_Air_Flow = auto()
       FCU_Zone_Temperature = auto()
       FCU_Zone_Air_Temperature = auto()
       FCU_Supply_Fan_VFD_Speed = auto()
       FCU_Supply_Air_Temperature = auto()
       FCU_Run_Time = auto()
       FCU_Return_Fan_VFD_Speed = auto()
       FCU_Return_Air_Temperature = auto()
       FCU_Return_Air_Humidity = auto()
       FCU_Return_Air_CO2 = auto()
       FCU_Humidity = auto()
       FCU_Discharge_Fan_VFD_Speed = auto()
       FCU_Discharge_Air_Temperature = auto()
       FCU_Chilled_Valve_Pressure = auto()
       FCU_CO2 = auto()
       Exhaust_Fan_Piezoelectric = auto()
       Exhaust_Fan_Fan_Air_Flow = auto()
       Exhaust_Air_Velocity_Pressure = auto()
       Exhaust_Air_Temperature = auto()
       Exhaust_Air_Static_Pressure = auto()
       Exhaust_Air_Stack_Flow = auto()
       Exhaust_Air_Humidity = auto()
       Exhaust_Air_Flow = auto()
       Environment_Box_Temperature = auto()
       Enthalpy = auto()
       Entering_Water_Temperature = auto()
       Energy = auto()
       Electricity_Power = auto()
       Electricity_Energy = auto()
       Drive_Temperature = auto()
       Domestic_Hot_Water_Supply_Temperature = auto()
       Domestic_Hot_Water_Discharge_Temperature = auto()
       Discharge_Fan_VFD_Speed = auto()
       Discharge_Fan_Piezoelectric = auto()
       Discharge_Fan_Air_Flow = auto()
       Discharge_Air_Velocity_Pressure = auto()
       Discharge_Air_Temperature = auto()
       Discharge_Air_Static_Pressure = auto()
       Discharge_Air_Humidity = auto()
       Discharge_Air_Flow = auto()
       Direction = auto()
       Differential_Temperature = auto()
       Differential_Speed = auto()
       Differential_Pressure = auto()
       Dewpoint = auto()
       Demand = auto()
       Deionised_Water_Level = auto()
       Deionised_Water_Conductivity = auto()
       Damper_Position = auto()
       DI_Water_Level = auto()
       DHWS_Domestic_Hot_Water_Supply_Temperature = auto()
       DHWS_Domestic_Hot_Water_Discharge_Temperature = auto()
       DC_Bus_Voltage = auto()
       Current_Steam_Usage = auto()
       Current = auto()
       Cooling_Thermal_Power = auto()
       Cooling_Coil_Valve_Pressure = auto()
       Cooling_Coil_Supply_Air_Temperature = auto()
       Cooling_Coil_Discharge_Air_Temperature = auto()
       Conductivity = auto()
       Condensor_Temperature = auto()
       Condenser_Condensor_Temperature = auto()
       Coldest_Zone_Temperature = auto()
       Cold_Box_Temperature = auto()
       Chiller_Run_Time = auto()
       Chiller_On_Timer = auto()
       Chilled_Water_Usage = auto()
       Chilled_Water_Temperature_Differential = auto()
       Chilled_Water_Supply_Temperature = auto()
       Chilled_Water_Supply_Flow = auto()
       Chilled_Water_Return_Temperature = auto()
       Chilled_Water_Flow = auto()
       Chilled_Water_Discharge_Temperature = auto()
       Chilled_Water_Discharge_Flow = auto()
       Chilled_Water_Differential_Pressure = auto()
       Chilled_Valve_Pressure = auto()
       Capacity = auto()
       CWS_Chilled_Water_Temperature_Differential = auto()
       CWS_Chilled_Water_Supply_Temperature = auto()
       CWS_Chilled_Water_Supply_Flow = auto()
       CWS_Chilled_Water_Return_Temperature = auto()
       CWS_Chilled_Water_Discharge_Temperature = auto()
       CWS_Chilled_Water_Discharge_Flow = auto()
       CWS_Chilled_Water_Differential_Pressure = auto()
       CRAC_Zone_Humidity = auto()
       CRAC_Temperature = auto()
       CRAC_Humidity = auto()
       CRAC_Capacity = auto()
       CO2 = auto()
       CO2_Level = auto()
       CO2_Differential = auto()
       Bypass_Damper_Position = auto()
       Bypass_Air_Flow = auto()
       Building_Static_Pressure = auto()
       Booster_Fan_Air_Flow = auto()
       Boiler_Run_Time = auto()
       Battery_Voltage = auto()
       Average_Zone_Temperature = auto()
       Average_Supply_Air_Flow = auto()
       Average_Exhaust_Air_Static_Pressure = auto()
       Average_Discharge_Air_Flow = auto()
       Average_Cooling_Demand = auto()
       Angle = auto()
       Ambient_Illumination = auto()
       Air_Grains = auto()
       Air_Flow = auto()
       AHU_Zone_Temperature = auto()
       AHU_Zone_Air_Temperature = auto()
       AHU_Warmest_Zone_Temperature = auto()
       AHU_Underfloor_Temperature = auto()
       AHU_Trace_Heat = auto()
       AHU_Supply_Fan_VFD_Speed = auto()
       AHU_Supply_Fan_Piezoelectric = auto()
       AHU_Supply_Fan_Air_Flow = auto()
       AHU_Supply_Air_Temperature = auto()
       AHU_Supply_Air_Static_Pressure = auto()
       AHU_Supply_Air_Humidity = auto()
       AHU_Static_Pressure = auto()
       AHU_Run_Time = auto()
       AHU_Return_Fan_VFD_Speed = auto()
       AHU_Return_Fan_Differential_Speed = auto()
       AHU_Return_Fan_Air_Flow = auto()
       AHU_Return_Air_Temperature = auto()
       AHU_Return_Air_Humidity = auto()
       AHU_Return_Air_Grains = auto()
       AHU_Return_Air_Flow = auto()
       AHU_Return_Air_Enthalpy = auto()
       AHU_Return_Air_Dewpoint = auto()
       AHU_Return_Air_CO2 = auto()
       AHU_Preheat_Supply_Air_Temperature = auto()
       AHU_Preheat_Discharge_Air_Temperature = auto()
       AHU_Preheat_Coil_Entering_Air_Temperature = auto()
       AHU_Outside_Air_Temperature = auto()
       AHU_Outside_Air_Lockout_Temperature_Differential = auto()
       AHU_Outside_Air_Humidity = auto()
       AHU_Outside_Air_Grains = auto()
       AHU_Outside_Air_Flow = auto()
       AHU_Outside_Air_Enthalpy = auto()
       AHU_Outside_Air_Dewpoint = auto()
       AHU_Outside_Air_Damper_Position = auto()
       AHU_Outside_Air_CO2 = auto()
       AHU_Mixed_Air_Temperature = auto()
       AHU_Mixed_Air_Damper_Position = auto()
       AHU_Lowest_Zone_Temperature = auto()
       AHU_Lowest_Exhaust_Air_Static_Pressure = auto()
       AHU_Low_Outside_Air_Temperature_Enable_Differential = auto()
       AHU_Highest_Zone_Temperature = auto()
       AHU_Heat_Wheel_Voltage = auto()
       AHU_Heat_Wheel_Supply_Air_Temperature = auto()
       AHU_Heat_Wheel_Speed = auto()
       AHU_Heat_Wheel_Discharge_Air_Temperature = auto()
       AHU_Heat_Wheel_Differential_Pressure = auto()
       AHU_Frost = auto()
       AHU_Exhaust_Fan_Piezoelectric = auto()
       AHU_Exhaust_Air_Velocity_Pressure = auto()
       AHU_Exhaust_Air_Temperature = auto()
       AHU_Exhaust_Air_Static_Pressure = auto()
       AHU_Exhaust_Air_Stack_Flow = auto()
       AHU_Exhaust_Air_Humidity = auto()
       AHU_Exhaust_Air_Flow = auto()
       AHU_Discharge_Fan_VFD_Speed = auto()
       AHU_Discharge_Fan_Piezoelectric = auto()
       AHU_Discharge_Fan_Air_Flow = auto()
       AHU_Discharge_Air_Temperature = auto()
       AHU_Discharge_Air_Static_Pressure = auto()
       AHU_Discharge_Air_Humidity = auto()
       AHU_Differential_Pressure = auto()
       AHU_Cooling_Coil_Supply_Air_Temperature = auto()
       AHU_Cooling_Coil_Discharge_Air_Temperature = auto()
       AHU_Coldest_Zone_Temperature = auto()
       AHU_CO2_Differential = auto()
       AHU_Bypass_Damper_Position = auto()
       AHU_Bypass_Air_Flow = auto()
       AHU_Building_Static_Pressure = auto()
       AHU_Average_Zone_Temperature = auto()
       AHU_Average_Exhaust_Air_Static_Pressure = auto()   
